@@ -1,19 +1,19 @@
 #
-# This is an extension to the Thunar file manager to allow better 
+# This is an extension to the Thunar file manager to allow better
 # integration with the Subversion source control system.
-# 
+#
 # Copyright (C) 2008-2008 by Adam Plumb <adamplumb@gmail.com>
-# 
+#
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # RabbitVCS is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
 #
@@ -21,7 +21,7 @@
 """
 
 Our module for everything related to the Thunar extension.
-  
+
 """
 
 from __future__ import with_statement
@@ -32,24 +32,29 @@ from os.path import isdir, isfile, realpath, basename
 import datetime
 import time
 import threading
-import urllib
-import thunarx
-import pysvn
-import gobject
-import gtk
+
+from rabbitvcs.util import helper
+
+import gi
+gi.require_version("Gtk", "3.0")
+sa = helper.SanitizeArgv()
+from gi.repository import GObject, Gtk, Thunarx
+sa.restore()
 
 from rabbitvcs.vcs.svn import SVN
 
+import os
+
 import rabbitvcs.ui
 import rabbitvcs.ui.property_page
 from rabbitvcs.util.helper import launch_ui_window, launch_diff_tool
 from rabbitvcs.util.helper import get_file_extension, get_common_directory
 from rabbitvcs.util.helper import pretty_timedelta
+from rabbitvcs.util.strings import S
 from rabbitvcs.util.decorators import timeit, disable
-from rabbitvcs.util.contextmenu import MainContextMenu, SEPARATOR
+from rabbitvcs.util.contextmenu import MenuBuilder, MainContextMenu, SEPARATOR
 
 from rabbitvcs.util.log import Log, reload_log_settings
-import six
 log = Log("rabbitvcs.util.extensions.thunarx.RabbitVCS")
 
 from rabbitvcs import gettext
@@ -61,17 +66,17 @@ settings = SettingsManager()
 import rabbitvcs.services.service
 from rabbitvcs.services.checkerservice import StatusCheckerStub as StatusChecker
 
-class RabbitVCS(thunarx.MenuProvider, thunarx.PropertyPageProvider):
-    """ 
-    This is the main class that implements all of our awesome features.
-    
+class RabbitVCS(GObject.GObject, Thunarx.MenuProvider, Thunarx.PropertyPageProvider):
     """
-    
+    This is the main class that implements all of our awesome features.
+
+    """
+
     #: Maps statuses to emblems.
     #: TODO: should probably be possible to create this dynamically
     EMBLEMS = rabbitvcs.ui.STATUS_EMBLEMS
-    
-    #: A list of statuses which count as modified (for a directory) in 
+
+    #: A list of statuses which count as modified (for a directory) in
     #: TortoiseSVN emblem speak.
     MODIFIED_STATUSES = [
         SVN.STATUS["added"],
@@ -80,51 +85,51 @@ class RabbitVCS(thunarx.MenuProvider, thunarx.PropertyPageProvider):
         SVN.STATUS["modified"],
         SVN.STATUS["missing"]
     ]
-    
+
     MODIFIED_TEXT_STATUSES = [
-        "added", 
+        "added",
         "deleted",
         "replaced",
         "modified",
         "missing"
     ]
-    
+
     #: This is our lookup table for C{NautilusVFSFile}s which we need for attaching
     #: emblems. This is mostly a workaround for not being able to turn a path/uri
-    #: into a C{NautilusVFSFile}. It looks like:::
-    #: 
-    #:     nautilusVFSFile_table = {
+    #: into a C{VFSFile}. It looks like:::
+    #:
+    #:     VFSFile_table = {
     #:        "/foo/bar/baz": <NautilusVFSFile>
-    #:     
+    #:
     #:     }
-    #: 
+    #:
     #: Keeping track of C{NautilusVFSFile}s is a little bit complicated because
     #: when an item is moved (renamed) C{update_file_info} doesn't get called. So
     #: we also add C{NautilusVFSFile}s to this table from C{get_file_items} etc.
-    nautilusVFSFile_table = {}
-    
+    VFSFile_table = {}
+
     #: Without an actual status monitor it's not possible to just keep
     #: track of stuff that happens (e.g. a commit happens, files are added,
     #: such things). So at the moment we just add all interesting items
     #: to this list.
     monitored_files = []
-    
+
     #: This is in case we want to permanently enable invalidation of the status
     #: checker info. We put a path here before we invalidate the item, so that
     #: we don't enter an endless loop when updating the status.
     #: The callback should acquire this lock when pushing the path to this.
     always_invalidate = True
-    
+
     #: When we get the statuses from the callback, but them here for further
     #: use. There is a possible memory problem here if we put a lot of data in
     #: this - even when it's removed, Python may not release the memory. I do
     #: not know this for sure.
     #: This is of the form: [("path/to", {...status dict...}), ...]
     paths_from_callback = []
-    
+
     paths_last_lookup = {}
     paths_lookup_timeout = 30
-    
+
     #: It appears that the "update_file_info" call that is triggered by the
     #: "invalidate_extension_info" in the callback function happens
     #: synchronously (ie. in the same thread). However, given the nature of the
@@ -133,7 +138,7 @@ class RabbitVCS(thunarx.MenuProvider, thunarx.PropertyPageProvider):
     #: change in the future? Who knows. This should work for both the current
     #: situation, and the possibility that they are asynchronous.
     callback_paths_lock = threading.RLock()
-    
+
     #: A list of statuses that we want to keep track of for when a process
     #: might have done something.
     STATUSES_TO_MONITOR = copy.copy(MODIFIED_TEXT_STATUSES)
@@ -144,112 +149,114 @@ class RabbitVCS(thunarx.MenuProvider, thunarx.PropertyPageProvider):
         # checkout likely hasn't completed yet and the status will be:
         "incomplete"
     ])
-    
+
     def __init__(self):
         threading.currentThread().setName("RabbitVCS extension thread")
-        
+
         self.status_checker = StatusChecker()
-    
+
     def get_local_path(self, item):
-        return urllib.unquote(item.get_uri().replace("file://", ""))
+        if item.get_uri_scheme() != "file":
+            return None
+        return item.get_location().get_path()
 
     #~ @disable
     # @timeit
-    def get_file_actions(self, window, items):
+    def get_file_menu_items(self, window, items):
         """
         Menu activated with items selected. Nautilus also calls this function
         when rendering submenus, even though this is not needed since the entire
         menu has already been returned.
-        
-        Note that calling C{nautilusVFSFile.invalidate_extension_info()} will 
+
+        Note that calling C{nautilusVFSFile.invalidate_extension_info()} will
         also cause get_file_items to be called.
-        
+
         @type   window: NautilusNavigationWindow
         @param  window:
-        
+
         @type   items:  list of NautilusVFSFile
         @param  items:
-        
+
         @rtype:         list of MenuItems
         @return:        The context menu entries to add to the menu.
-        
+
         """
 
         paths = []
         for item in items:
             if self.valid_uri(item.get_uri()):
-                path = realpath(six.text_type(self.get_local_path(item), "utf-8"))
+                path = realpath(S(self.get_local_path(item)))
                 paths.append(path)
-                self.nautilusVFSFile_table[path] = item
+                self.VFSFile_table[path] = item
 
         if len(paths) == 0: return []
-        
-        return ThunarxMainContextMenu(self, window.get_data("base_dir"), paths).get_menu()
-    
+
+        return ThunarxMainContextMenu(self, window.base_dir, paths).get_menu()
+
     #~ @disable
     @timeit
-    def get_folder_actions(self, window, item):
+    def get_folder_menu_items(self, window, item):
         """
         Menu activated on entering a directory. Builds context menu for File
         menu and for window background.
-        
+
         @type   window: NautilusNavigationWindow
         @param  window:
-        
+
         @type   item:   NautilusVFSFile
         @param  item:
-        
+
         @rtype:         list of MenuItems
         @return:        The context menu entries to add to the menu.
-        
+
         """
-        
+
         if not self.valid_uri(item.get_uri()): return
-        path = realpath(six.text_type(self.get_local_path(item), "utf-8"))
-        self.nautilusVFSFile_table[path] = item
-        
+        path = realpath(S(self.get_local_path(item)))
+        self.VFSFile_table[path] = item
+
         # log.debug("get_background_items() called")
-        
-        window.set_data("base_dir", path)
-        
+
+        window.base_dir = path
+
         return ThunarxMainContextMenu(self, path, [path]).get_menu()
-            
+
     #
     # Helper functions
     #
-    
+
     def valid_uri(self, uri):
         """
         Check whether or not it's a good idea to have RabbitVCS do
         its magic for this URI. Some examples of URI schemes:
-        
+
         x-nautilus-desktop:/// # e.g. mounted devices on the desktop
-        
+
         """
-        
+
         if not uri.startswith("file://"): return False
-        
+
         return True
-    
+
     #
     # Some methods to help with keeping emblems up-to-date
     #
-    
+
     def rescan_after_process_exit(self, proc, paths):
-        """ 
+        """
         Rescans all of the items on our C{monitored_files} list after the
         process specified by C{proc} completes. Also checks the paths
         that were passed.
-        
+
         TODO: the monitored_files list could grow quite large if somebody
         browses a lot of working copies. It probably won't affect anything
         (most importantly performance) all that negatively.
-        
+
         """
-        
+
         def do_check():
             # We'll check the paths first (these were the paths that
-            # were originally passed along to the context menu). 
+            # were originally passed along to the context menu).
             #
             # This is needed among other things for:
             #
@@ -261,9 +268,9 @@ class RabbitVCS(thunarx.MenuProvider, thunarx.PropertyPageProvider):
                                                  recurse=True,
                                                  invalidate=True,
                                                  summary=True)
-            
+
         self.execute_after_process_exit(proc, do_check)
-        
+
     def execute_after_process_exit(self, proc, func=None):
 
         def is_process_still_alive():
@@ -271,37 +278,37 @@ class RabbitVCS(thunarx.MenuProvider, thunarx.PropertyPageProvider):
             # First we need to see if the commit process is still running
 
             retval = proc.poll()
-            
+
             log.debug("%s" % retval)
-            
+
             still_going = (retval is None)
 
             if not still_going and callable(func):
                 func()
-            
+
             return still_going
 
         # Add our callback function on a 1 second timeout
-        gobject.timeout_add_seconds(1, is_process_still_alive)
-        
-    # 
+        GObject.timeout_add_seconds(1, is_process_still_alive)
+
+    #
     # Some other methods
-    # 
-    
+    #
+
     def reload_settings(self, proc):
         """
         Used to re-load settings after the settings dialog has been closed.
-        
+
         FIXME: This probably doesn't belong here, ideally the settings manager
-        does this itself and make sure everything is reloaded properly 
+        does this itself and make sure everything is reloaded properly
         after the settings dialogs saves.
         """
-    
+
         def do_reload_settings():
             globals()["settings"] = SettingsManager()
             globals()["log"] = reload_log_settings()("rabbitvcs.util.extensions.thunar")
             log.debug("Re-scanning settings")
-            
+
         self.execute_after_process_exit(proc, do_reload_settings)
 
     def get_property_pages(self, items):
@@ -309,38 +316,39 @@ class RabbitVCS(thunarx.MenuProvider, thunarx.PropertyPageProvider):
         paths = []
         for item in items:
             if self.valid_uri(item.get_uri()):
-                path = realpath(six.text_type(self.get_local_path(item), "utf-8"))
+                path = realpath(S(self.get_local_path(item)))
                 paths.append(path)
-                self.nautilusVFSFile_table[path] = item
+                self.VFSFile_table[path] = item
 
         if len(paths) == 0: return []
 
         label = rabbitvcs.ui.property_page.PropertyPageLabel().get_widget()
         page = rabbitvcs.ui.property_page.PropertyPage(paths).get_widget()
-        
-        ppage = thunarx.PropertyPage("")
+
+        ppage = Thunarx.PropertyPage.new("")
         ppage.set_label_widget(label)
-        ppage.add(page)        
-        
+        ppage.add(page)
+
         return [ppage]
 
 from rabbitvcs.util.contextmenuitems import *
 
-class ThunarxContextMenu(rabbitvcs.util.contextmenu.MenuBuilder):
+class ThunarxContextMenu(MenuBuilder):
     """
-    Provides a standard Gtk Context Menu class used for all context menus
-    in gtk dialogs/windows.
-    
+    Provides a standard Thunar context menu (ie. a list of
+    "Thunarx.MenuItem"s).
     """
-    
+
     signal = "activate"
-        
+
     def make_menu_item(self, item, id_magic):
-        return item.make_thunar_action(id_magic)
-    
+        return item.make_thunarx_menu_item(id_magic)
+
     def attach_submenu(self, menu_node, submenu_list):
-		menu_node.set_sub_actions(submenu_list)
-    
+        submenu = Thunarx.Menu()
+        menu_node.set_menu(submenu)
+        [submenu.append_item(item) for item in submenu_list]
+
     def top_level_menu(self, items):
         return items
 
