@@ -27,23 +27,24 @@ import threading
 from os.path import basename
 
 import shutil
-import pygtk
-import gobject
-import gtk
+import gi
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gtk, GObject, Gdk
 
 from rabbitvcs.ui import InterfaceView
 import rabbitvcs.ui.widget
 import rabbitvcs.ui.dialog
 import rabbitvcs.util
 import rabbitvcs.vcs
-import rabbitvcs.util.helper
+from rabbitvcs.util import helper
+from rabbitvcs.util.strings import S
 from rabbitvcs.ui.dialog import MessageBox
 from rabbitvcs.util.decorators import gtk_unsafe
 
 from rabbitvcs import gettext
 _ = gettext.gettext
 
-gobject.threads_init()
+helper.gobject_threads_init()
 
 from rabbitvcs.util.log import Log
 log = Log("rabbitvcs.ui.action")
@@ -76,7 +77,7 @@ class VCSNotifier(InterfaceView):
     def focus_on_ok_button(self):
         pass
 
-class DummyNotifier:
+class DummyNotifier(object):
     def __init__(self):
         pass
 
@@ -116,7 +117,7 @@ class MessageCallbackNotifier(VCSNotifier):
 
         self.table = rabbitvcs.ui.widget.Table(
             self.get_widget("table"),
-            [gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING],
+            [GObject.TYPE_STRING, GObject.TYPE_STRING, GObject.TYPE_STRING],
             [_("Action"), _("Path"), _("Mime Type")]
         )
 
@@ -146,52 +147,33 @@ class MessageCallbackNotifier(VCSNotifier):
     def on_ok_clicked(self, widget):
         self.close()
 
+    @gtk_unsafe
     def toggle_ok_button(self, sensitive):
-        if not self.client_in_same_thread:
-            gtk.gdk.threads_enter()
-
         self.finished = True
         self.get_widget("ok").set_sensitive(sensitive)
         self.get_widget("saveas").set_sensitive(sensitive)
 
-        if not self.client_in_same_thread:
-            gtk.gdk.threads_leave()
-
+    @gtk_unsafe
     def append(self, entry):
-        if not self.client_in_same_thread:
-            gtk.gdk.threads_enter()
-
         self.table.append(entry)
         self.table.scroll_to_bottom()
 
-        if not self.client_in_same_thread:
-            gtk.gdk.threads_leave()
-
     def get_title(self):
         return self.get_widget("Notification").get_title()
 
+    @gtk_unsafe
     def set_title(self, title):
-        if not self.client_in_same_thread:
-            gtk.gdk.threads_enter()
-
         self.get_widget("Notification").set_title(title)
 
-        if not self.client_in_same_thread:
-            gtk.gdk.threads_leave()
-
+    @gtk_unsafe
     def set_header(self, header):
         self.set_title(header)
 
-        if not self.client_in_same_thread:
-            gtk.gdk.threads_enter()
-
         self.get_widget("action").set_markup(
             "<span size=\"xx-large\"><b>%s</b></span>" % header
         )
 
-        if not self.client_in_same_thread:
-            gtk.gdk.threads_leave()
-
+    @gtk_unsafe
     def focus_on_ok_button(self):
         self.get_widget("ok").grab_focus()
 
@@ -205,6 +187,7 @@ class MessageCallbackNotifier(VCSNotifier):
     def enable_saveas(self):
         self.get_widget("saveas").set_sensitive(True)
 
+    @gtk_unsafe
     def disable_saveas(self):
         self.get_widget("saveas").set_sensitive(False)
 
@@ -269,6 +252,8 @@ class VCSAction(threading.Thread):
     def __init__(self, client, register_gtk_quit=False, notification=True,
             run_in_thread=True):
 
+        self.run_in_thread = run_in_thread
+
         if run_in_thread is True:
             threading.Thread.__init__(self)
 
@@ -299,11 +284,17 @@ class VCSAction(threading.Thread):
         self.pbar_ticks = None
         self.pbar_ticks_current = -1
 
-        # Tells the notification window to do a gtk.main_quit() when closing
+        # Tells the notification window to do a Gtk.main_quit() when closing
         # Is used when the script is run from a command line
         if register_gtk_quit:
             self.notification.register_gtk_quit()
 
+    def schedule(self):
+        if self.run_in_thread:
+            self.start()
+        else:
+            self.run()
+
     def set_pbar_ticks(self, num):
         """
         Set the total number of ticks to represent in the progress bar.
@@ -319,20 +310,14 @@ class VCSAction(threading.Thread):
     def set_progress_fraction(self, fraction):
         """
         An alternative method to access the progress bar directly.
-        
+
         @type   percentage: int
         @param  percentage: The percentage value to set the progress bar.
 
         """
-        
+
         if self.has_notifier:
-            if not self.client_in_same_thread:
-                gtk.gdk.threads_enter()
-    
             self.notification.pbar.update(fraction)
-    
-            if not self.client_in_same_thread:
-                gtk.gdk.threads_leave()
 
     def set_header(self, header):
         self.notification.set_header(header)
@@ -398,16 +383,19 @@ class VCSAction(threading.Thread):
 
         """
 
-        if self.message is None:
-            gtk.gdk.threads_enter()
-            dialog = rabbitvcs.ui.dialog.TextChange(_("Log Message"))
-            result = dialog.run()
-            gtk.gdk.threads_leave()
-
-            should_continue = (result[0] == gtk.RESPONSE_OK)
-            return should_continue, result[1].encode("utf-8")
-        else:
-            return True, self.message.encode("utf-8")
+        should_continue = True
+        message = self.message
+        if message is None:
+            settings = rabbitvcs.util.settings.SettingsManager()
+            message = settings.get_multiline("general", "default_commit_message")
+            result = helper.run_in_main_thread(lambda: rabbitvcs.ui.dialog.TextChange(_("Log Message"), message).run())
+            should_continue = (result[0] == Gtk.ResponseType.OK)
+            message = result[1]
+        if isinstance(message, bytes):
+            message = message.decode()
+        if not should_continue:
+            self.set_cancel()
+        return should_continue, message
 
     def get_login(self, realm, username, may_save):
         """
@@ -432,19 +420,12 @@ class VCSAction(threading.Thread):
 
         @rtype:             (boolean, string, string, boolean)
         @return:            (True=continue/False=cancel, username,password, may_save)
-
         """
 
         if self.login_tries >= 3:
             return (False, "", "", False)
 
-        gtk.gdk.threads_enter()
-        dialog = rabbitvcs.ui.dialog.Authentication(
-            realm,
-            may_save
-        )
-        result = dialog.run()
-        gtk.gdk.threads_leave()
+        result = helper.run_in_main_thread(lambda: rabbitvcs.ui.dialog.Authentication(realm, may_save).run())
 
         if result is not None:
             self.login_tries += 1
@@ -466,24 +447,18 @@ class VCSAction(threading.Thread):
 
         @rtype:         (boolean, int, boolean)
         @return:        (True=Accept/False=Deny, number of accepted failures, remember)
-
         """
 
-        gtk.gdk.threads_enter()
-
-        if not data:
-            return (False, 0, False)
-
-        dialog = rabbitvcs.ui.dialog.Certificate(
-            data["realm"],
-            data["hostname"],
-            data["issuer_dname"],
-            data["valid_from"],
-            data["valid_until"],
-            data["finger_print"]
-        )
-        result = dialog.run()
-        gtk.gdk.threads_leave()
+        result = 0
+        if data:
+            result = helper.run_in_main_thread(lambda: rabbitvcs.ui.dialog.Certificate(
+                data["realm"],
+                data["hostname"],
+                data["issuer_dname"],
+                data["valid_from"],
+                data["valid_until"],
+                data["finger_print"]
+            ).run())
 
         if result == 0:
             #Deny
@@ -510,18 +485,12 @@ class VCSAction(threading.Thread):
 
         @rtype:             (boolean, string, boolean)
         @return:            (True=continue/False=cancel, password, may save)
-
         """
 
-        gtk.gdk.threads_enter()
-        dialog = rabbitvcs.ui.dialog.CertAuthentication(
-            realm,
-            may_save
-        )
-        result = dialog.run()
-        gtk.gdk.threads_leave()
-
-        return result
+        return helper.run_in_main_thread(lambda: rabbitvcs.ui.dialog.CertAuthentication(
+                realm,
+                may_save
+            ).run())
 
     def get_client_cert(self, realm, may_save):
         """
@@ -538,18 +507,12 @@ class VCSAction(threading.Thread):
 
         @rtype:             (boolean, string, boolean)
         @return:            (True=continue/False=cancel, password, may save)
-
         """
 
-        gtk.gdk.threads_enter()
-        dialog = rabbitvcs.ui.dialog.SSLClientCertPrompt(
-            realm,
-            may_save
-        )
-        result = dialog.run()
-        gtk.gdk.threads_leave()
-
-        return result
+        return helper.run_in_main_thread(lambda: rabbitvcs.ui.dialog.SSLClientCertPrompt(
+                realm,
+                may_save
+            ).run())
 
     def set_log_message(self, message):
         """
@@ -558,7 +521,6 @@ class VCSAction(threading.Thread):
 
         @type   message: string
         @param  message: Set a log message.
-
         """
 
         self.message = message
@@ -574,11 +536,10 @@ class VCSAction(threading.Thread):
 
         @type   message: string
         @param  message: A status message.
-
         """
 
         if message is not None:
-            self.notification.get_widget("status").set_text(message)
+            self.notification.get_widget("status").set_text(S(message).display())
 
     def append(self, func, *args, **kwargs):
         """
@@ -705,7 +666,7 @@ class SVNAction(VCSAction):
 
             if (is_known_action
                     and is_complete_action
-                    and "revision" in data 
+                    and "revision" in data
                     and data["revision"]):
                 self.notification.append(
                     ["", "Revision %s" % data["revision"].number, ""]
@@ -723,7 +684,7 @@ class SVNAction(VCSAction):
             self.queue.insert(position+1, self.edit_conflict, data)
 
     def edit_conflict(self, data):
-        rabbitvcs.util.helper.launch_ui_window("editconflicts", [data["path"]], block=True)
+        helper.launch_ui_window("editconflicts", [data["path"]], block=True)
 
 class GitAction(VCSAction):
     def __init__(self, client, register_gtk_quit=False, notification=True,
@@ -755,17 +716,12 @@ class GitAction(VCSAction):
                     self.notification.append(["", data, ""])
 
     def get_user(self):
-        gtk.gdk.threads_enter()
-        dialog = rabbitvcs.ui.dialog.NameEmailPrompt()
-        result = dialog.run()
-        gtk.gdk.threads_leave()
-
-        return result
+        return helper.run_in_main_thread(lambda: rabbitvcs.ui.dialog.NameEmailPrompt().run())
 
     def conflict_filter(self, data):
         if str(data).startswith("ERROR:"):
             path = data[27:]
-            rabbitvcs.util.helper.launch_ui_window("editconflicts", [path], block=True)
+            helper.launch_ui_window("editconflicts", [path], block=True)
 
 class MercurialAction(VCSAction):
     def __init__(self, client, register_gtk_quit=False, notification=True,
@@ -795,17 +751,12 @@ class MercurialAction(VCSAction):
                     self.notification.append(["", data, ""])
 
     def get_user(self):
-        gtk.gdk.threads_enter()
-        dialog = rabbitvcs.ui.dialog.NameEmailPrompt()
-        result = dialog.run()
-        gtk.gdk.threads_leave()
-
-        return result
+        return helper.run_in_main_thread(lambda: rabbitvcs.ui.dialog.NameEmailPrompt().run())
 
     def conflict_filter(self, data):
         if str(data).startswith("ERROR:"):
             path = data[27:]
-            rabbitvcs.util.helper.launch_ui_window("editconflicts", [path], block=True)
+            helper.launch_ui_window("editconflicts", [path], block=True)
 
 def vcs_action_factory(client, register_gtk_quit=False, notification=True,
         run_in_thread=True):
