@@ -15,16 +15,17 @@ __author__    = "Scott James Remnant <scott@ubuntu.com>"
 
 import math
 
-import gtk
-import gobject
-import pango
+import gi
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gtk, GObject, Pango
+
 import cairo
 
 # Styles used when rendering revision graph edges
 style_SOLID = 0
 style_DASHED = 1
 
-class CellRendererGraph(gtk.GenericCellRenderer):
+class CellRendererGraph(Gtk.CellRenderer):
     """Cell renderer for directed graph.
 
     Properties:
@@ -33,12 +34,16 @@ class CellRendererGraph(gtk.GenericCellRenderer):
       out_lines         (start, end, colour, style) tuple list to draw outward lines.
     """
 
-    columns_len = 0
+    columns_len = 1
+    _box_size = None
+    node = None
+    in_lines = []
+    out_lines = []
 
     __gproperties__ = {
-        "graph":         ( gobject.TYPE_PYOBJECT, "graph",
+        "graph":         ( GObject.TYPE_PYOBJECT, "graph",
                           "revision node instruction",
-                          gobject.PARAM_WRITABLE
+                          GObject.PARAM_WRITABLE
                         )
         }
 
@@ -49,7 +54,9 @@ class CellRendererGraph(gtk.GenericCellRenderer):
             return
 
         if property.name == "graph":
-            (self.node, self.in_lines, self.out_lines) = value
+            self.node = value["node"]
+            self.in_lines = value["in_lines"]
+            self.out_lines = value["out_lines"]
         else:
             raise AttributeError("no such property: '%s'" % property.name)
 
@@ -59,18 +66,8 @@ class CellRendererGraph(gtk.GenericCellRenderer):
         Cache this as it's probably expensive to get.  It ensures that we
         draw the graph at least as large as the text.
         """
-        try:
-            return self._box_size
-        except AttributeError:
-            pango_ctx = widget.get_pango_context()
-            font_desc = widget.get_style().font_desc
-            metrics = pango_ctx.get_metrics(font_desc)
 
-            ascent = pango.PIXELS(metrics.get_ascent())
-            descent = pango.PIXELS(metrics.get_descent())
-
-            self._box_size = ascent + descent + 1
-            return self._box_size
+        return 20
 
     def set_colour(self, ctx, colour, bg, fg):
         """Set the context source colour.
@@ -86,9 +83,9 @@ class CellRendererGraph(gtk.GenericCellRenderer):
             colour_rgb = int(r, 16) / 255., int(g, 16) / 255., int(b, 16) / 255.
         else:
             if colour == 0:
-                colour_rgb = gtklib.MAINLINE_COLOR
+                colour_rgb = Gtklib.MAINLINE_COLOR
             else:
-                colour_rgb = gtklib.LINE_COLORS[colour % len(gtklib.LINE_COLORS)]
+                colour_rgb = Gtklib.LINE_COLORS[colour % len(Gtklib.LINE_COLORS)]
 
         red   = (colour_rgb[0] * fg) or bg
         green = (colour_rgb[1] * fg) or bg
@@ -96,13 +93,14 @@ class CellRendererGraph(gtk.GenericCellRenderer):
 
         ctx.set_source_rgb(red, green, blue)
 
-    def on_get_size(self, widget, cell_area):
+    def do_get_size(self, widget, cell_area):
         """Return the size we need for this cell.
 
         Each cell is drawn individually and is only as wide as it needs
         to be, we let the TreeViewColumn take care of making them all
         line up.
         """
+
         box_size = self.box_size(widget) + 1
 
         width = box_size * (self.columns_len + 1)
@@ -111,7 +109,7 @@ class CellRendererGraph(gtk.GenericCellRenderer):
         # FIXME I have no idea how to use cell_area properly
         return (0, 0, width, height)
 
-    def on_render(self, window, widget, bg_area, cell_area, exp_area, flags):
+    def do_render(self, ctx, widget, bg_area, cell_area, flags):
         """Render an individual cell.
 
         Draws the cell contents using cairo, taking care to clip what we
@@ -124,8 +122,9 @@ class CellRendererGraph(gtk.GenericCellRenderer):
         to cross other columns we actually draw it as in the .---' style
         instead of a pure diagonal ... this reduces confusion by an
         incredible amount.
+
         """
-        ctx = window.cairo_create()
+
         ctx.rectangle(bg_area.x, bg_area.y, bg_area.width, bg_area.height)
         ctx.clip()
 
@@ -134,12 +133,12 @@ class CellRendererGraph(gtk.GenericCellRenderer):
         # Maybe draw branch head highlight under revision node
         if self.node:
             (column, colour) = self.node
-            arc_start_position_x = cell_area.x + box_size * column + box_size / 2; 
+            arc_start_position_x = cell_area.x + box_size * column + box_size / 2;
             arc_start_position_y = cell_area.y + cell_area.height / 2;
 
         ctx.set_line_width(box_size / 8)
         ctx.set_line_cap(cairo.LINE_CAP_ROUND)
-        
+
         # Draw lines into the cell
         if self.in_lines:
             for start, end, lcolour in self.in_lines:
@@ -166,16 +165,17 @@ class CellRendererGraph(gtk.GenericCellRenderer):
         ctx.stroke_preserve()
         self.set_colour(ctx, colour, 0.5, 1.0)
         ctx.fill()
+        ctx.save()
 
     def render_line (self, ctx, cell_area, box_size, mid,
             height, start, end, colour, style):
+
         if start is None:
             x = cell_area.x + box_size * end + box_size / 2
             ctx.move_to(x, mid + height / 3)
             ctx.line_to(x, mid + height / 3)
             ctx.move_to(x, mid + height / 6)
             ctx.line_to(x, mid + height / 6)
-
         elif end is None:
             x = cell_area.x + box_size * start + box_size / 2
             ctx.move_to(x, mid - height / 3)
