@@ -1,29 +1,33 @@
 from __future__ import absolute_import
 #
-# This is an extension to the Nautilus file manager to allow better 
+# This is an extension to the Nautilus file manager to allow better
 # integration with the Subversion source control system.
-# 
+#
 # Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
 # Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
 # Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
-# 
+#
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # RabbitVCS is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import pygtk
-import gobject
-import gtk
+from rabbitvcs.util import helper
+
+import gi
+gi.require_version("Gtk", "3.0")
+sa = helper.SanitizeArgv()
+from gi.repository import Gtk, GObject, Gdk
+sa.restore()
 
 from rabbitvcs.ui import InterfaceNonView, InterfaceView
 from rabbitvcs.ui.action import SVNAction, GitAction
@@ -36,9 +40,9 @@ _ = gettext.gettext
 class SVNUpdate(InterfaceNonView):
     """
     This class provides an interface to generate an "update".
-    Pass it a path and it will start an update, running the notification dialog.  
+    Pass it a path and it will start an update, running the notification dialog.
     There is no glade .
-    
+
     """
 
     def __init__(self, paths):
@@ -49,21 +53,22 @@ class SVNUpdate(InterfaceNonView):
     def start(self):
         self.action = SVNAction(
             self.svn,
-            register_gtk_quit=self.gtk_quit_is_set()
+            register_gtk_quit=self.gtk_quit_is_set(),
+            run_in_thread=False
         )
         self.action.append(self.action.set_header, _("Update"))
         self.action.append(self.action.set_status, _("Updating..."))
         self.action.append(self.svn.update, self.paths)
         self.action.append(self.action.set_status, _("Completed Update"))
         self.action.append(self.action.finish)
-        self.action.start()
+        self.action.schedule()
 
 class GitUpdate(InterfaceView):
     """
     This class provides an interface to generate an "update".
-    Pass it a path and it will start an update, running the notification dialog.  
+    Pass it a path and it will start an update, running the notification dialog.
     There is no glade .
-    
+
     """
 
     def __init__(self, paths):
@@ -77,7 +82,7 @@ class GitUpdate(InterfaceView):
             self.get_widget("repository_container"),
             self.git
         )
-        
+
     def on_apply_changes_toggled(self, widget, data=None):
         self.get_widget("merge").set_sensitive(self.get_widget("apply_changes").get_active())
         self.get_widget("rebase").set_sensitive(self.get_widget("apply_changes").get_active())
@@ -97,17 +102,18 @@ class GitUpdate(InterfaceView):
 
         self.action = GitAction(
             self.git,
-            register_gtk_quit=self.gtk_quit_is_set()
+            register_gtk_quit=self.gtk_quit_is_set(),
+            run_in_thread=False
         )
         self.action.append(self.action.set_header, _("Update"))
         self.action.append(self.action.set_status, _("Updating..."))
 
         if apply_changes:
             if rebase:
-                git_function_params.append ("rebase")
+                git_function_params.append("rebase")
 
             if fetch_all:
-                git_function_params.append ("all")
+                git_function_params.append("all")
                 repository = ""
                 branch = ""
 
@@ -117,14 +123,14 @@ class GitUpdate(InterfaceView):
                 self.action.append(self.git.fetch_all)
             else:
                 self.action.append(self.git.fetch, repository, branch)
-                
+
         self.action.append(self.action.set_status, _("Completed Update"))
         self.action.append(self.action.finish)
-        self.action.start()
+        self.action.schedule()
 
 
 classes_map = {
-    rabbitvcs.vcs.VCS_SVN: SVNUpdate, 
+    rabbitvcs.vcs.VCS_SVN: SVNUpdate,
     rabbitvcs.vcs.VCS_GIT: GitUpdate
 }
 
@@ -132,6 +138,7 @@ def update_factory(paths):
     guess = rabbitvcs.vcs.guess(paths[0])
     return classes_map[guess["vcs"]](paths)
 
+
 if __name__ == "__main__":
     from rabbitvcs.ui import main
     (options, paths) = main(usage="Usage: rabbitvcs update [path1] [path2] ...")
@@ -140,4 +147,4 @@ if __name__ == "__main__":
     window.register_gtk_quit()
     if isinstance(window, SVNUpdate):
         window.start()
-    gtk.main()
+    Gtk.main()
