@@ -1,29 +1,33 @@
 from __future__ import absolute_import
 #
-# This is an extension to the Nautilus file manager to allow better 
+# This is an extension to the Nautilus file manager to allow better
 # integration with the Subversion source control system.
-# 
+#
 # Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
 # Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
 # Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
-# 
+#
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # RabbitVCS is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import pygtk
-import gobject
-import gtk
+from rabbitvcs.util import helper
+
+import gi
+gi.require_version("Gtk", "3.0")
+sa = helper.SanitizeArgv()
+from gi.repository import Gtk, GObject, Gdk
+sa.restore()
 
 from rabbitvcs.ui import InterfaceView
 import rabbitvcs.ui.action
@@ -37,7 +41,7 @@ class UpdateToRevision(InterfaceView):
     """
     This class provides an interface to update a working copy to a specific
     revision.  It has a glade .
-    
+
     """
 
     def __init__(self, path, revision=None):
@@ -75,12 +79,12 @@ class SVNUpdateToRevision(UpdateToRevision):
             self.svn,
             register_gtk_quit=self.gtk_quit_is_set()
         )
-        
+
         if rollback:
             self.action.append(self.action.set_header, _("Rollback To Revision"))
             self.action.append(self.action.set_status, _("Rolling Back..."))
             self.action.append(
-                self.svn.merge_ranges, 
+                self.svn.merge_ranges,
                 self.svn.get_repo_url(self.path),
                 [(self.svn.revision("HEAD").primitive(), revision.primitive())],
                 self.svn.revision("head"),
@@ -91,16 +95,16 @@ class SVNUpdateToRevision(UpdateToRevision):
             self.action.append(self.action.set_header, _("Update To Revision"))
             self.action.append(self.action.set_status, _("Updating..."))
             self.action.append(
-                self.svn.update, 
+                self.svn.update,
                 self.path,
                 revision=revision,
                 recurse=recursive,
                 ignore_externals=omit_externals
             )
             self.action.append(self.action.set_status, _("Completed Update"))
-            
+
         self.action.append(self.action.finish)
-        self.action.start()
+        self.action.schedule()
 
     def on_revision_changed(self, revision_selector):
         # Only allow rollback when a revision number is specified
@@ -113,7 +117,7 @@ class SVNUpdateToRevision(UpdateToRevision):
 class GitUpdateToRevision(UpdateToRevision):
     def __init__(self, path, revision):
         UpdateToRevision.__init__(self, path, revision)
-        
+
         self.get_widget("revision_label").set_text(_("What revision/branch do you want to checkout?"))
 
         self.git = self.vcs.git(path)
@@ -134,7 +138,7 @@ class GitUpdateToRevision(UpdateToRevision):
             self.git,
             register_gtk_quit=self.gtk_quit_is_set()
         )
-        
+
         self.action.append(self.action.set_header, _("Checkout"))
         self.action.append(self.action.set_status, _("Checking out %s..." % revision))
         self.action.append(
@@ -144,7 +148,7 @@ class GitUpdateToRevision(UpdateToRevision):
         )
         self.action.append(self.action.set_status, _("Completed Checkout"))
         self.action.append(self.action.finish)
-        self.action.start()
+        self.action.schedule()
 
     def on_revision_changed(self, revision_selector):
         pass
@@ -161,13 +165,14 @@ def updateto_factory(vcs, path, revision=None):
 
     return classes_map[vcs](path, revision)
 
+
 if __name__ == "__main__":
     from rabbitvcs.ui import main, REVISION_OPT, VCS_OPT
     (options, args) = main(
         [REVISION_OPT, VCS_OPT],
         usage="Usage: rabbitvcs updateto [path]"
     )
- 
+
     window = updateto_factory(options.vcs, args[0], revision=options.revision)
     window.register_gtk_quit()
-    gtk.main()
+    Gtk.main()
