@@ -24,24 +24,24 @@ from __future__ import absolute_import
 # SOFTWARE.
 
 # Python translation from wrapLabel.{cc|h} by Gian Mario Tagliaretti
+import gi
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gtk, GObject, Pango
 
-import gtk
-import gobject
-import pango
+from rabbitvcs.util.strings import S
 
-
-class WrapLabel(gtk.Label):
+class WrapLabel(Gtk.Label):
     __gtype_name__ = 'WrapLabel'
 
     def __init__(self, str=None):
-        gtk.Label.__init__(self)
+        Gtk.Label.__init__(self)
 
         self.__wrap_width = 0
         self.layout = self.get_layout()
-        self.layout.set_wrap(pango.WRAP_WORD_CHAR)
+        self.layout.set_wrap(Pango.WrapMode.WORD_CHAR)
 
         if str != None:
-            self.set_text(str)
+            self.set_text(S(str).display())
 
         self.set_alignment(0.0, 0.0)
 
@@ -52,22 +52,22 @@ class WrapLabel(gtk.Label):
         requisition.height = height
 
     def do_size_allocate(self, allocation):
-        gtk.Label.do_size_allocate(self, allocation)
+        Gtk.Label.do_size_allocate(self, allocation)
         self.__set_wrap_width(allocation.width)
 
     def set_text(self, str):
-        gtk.Label.set_text(self, str)
+        Gtk.Label.set_text(self, S(str).display())
         self.__set_wrap_width(self.__wrap_width)
 
     def set_markup(self, str):
-        gtk.Label.set_markup(self, str)
+        Gtk.Label.set_markup(self, str)
         self.__set_wrap_width(self.__wrap_width)
 
     def __set_wrap_width(self, width):
         if width == 0:
             return
         layout = self.get_layout()
-        layout.set_width(width * pango.SCALE)
+        layout.set_width(width * Pango.SCALE)
         if self.__wrap_width != width:
             self.__wrap_width = width
             self.queue_resize()
