@@ -1,21 +1,21 @@
 from __future__ import absolute_import
 #
-# This is an extension to the Nautilus file manager to allow better 
+# This is an extension to the Nautilus file manager to allow better
 # integration with the Subversion source control system.
-# 
+#
 # Copyright (C) 2010 by Jason Heeris <jason.heeris@gmail.com>
 # Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
-# 
+#
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # RabbitVCS is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
 #
@@ -23,12 +23,12 @@ from __future__ import absolute_import
 import os.path
 
 import os
-if "NAUTILUS_PYTHON_REQUIRE_GTK3" in os.environ and os.environ["NAUTILUS_PYTHON_REQUIRE_GTK3"]:
-    from gi.repository import Gtk as gtk
-else:
-    import gtk
-    
-import rabbitvcs.util.helper
+import gi
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gtk
+
+from rabbitvcs.util import helper
+from rabbitvcs.util.strings import S
 
 from rabbitvcs import gettext
 _ = gettext.gettext
@@ -46,56 +46,56 @@ class MenuItem(object):
     "abstract" (in the language of Java) - it makes no sense to instantiate it
     directly. If you want to define a new kind of menu item, you need to
     subclass it like so:
-    
-    class MenuPerformMagic:
+
+    class MenuPerformMagic(MenuItem):
         identifier = "RabbitVCS::Perform_Magic"
         label = _("Perform Magic")
         tooltip = _("Put on your robe and wizard hat")
-        icon = "rabbitvcs-wand" # or, say, gtk.STOCK_OPEN
-    
+        icon = "rabbitvcs-wand"
+
     There is some introspection magic that goes on to associate the items
     themselves with certain methods of a ContextMenuCondition object or a
     ContextMenuCallback object. This is done by looking at the identifier - the
     part of the identifier after "::" is converted to lowercase and the item
     looks for a method of that name (eg. in the example above,
     "perform_magic").
-    
+
     It is easy to override this, just define condition_name and callback_name
     to be what you need. If the item cannot find anything, it defaults to not
     assigning the callback and having the condition return False.
-    
+
     There a few ways to organise this (and maybe it would be better to have the
     GtkContextMenu class do it), but this is it for the moment.
     """
-    
+
     @staticmethod
     def default_condition(*args, **kwargs):
         return False
-    
+
     @staticmethod
     def make_default_name(identifier):
         return identifier.split(MenuItem.IDENTIFIER_SEPARATOR)[-1].lower()
-    
+
     IDENTIFIER_SEPARATOR = "::"
-    
+
     # These are all explicitly defined here to make it obvious what a subclass
     # needs to set up.
-    
+
     # This is relevant for GTK and Nautilus - they require unique identifiers
     # for all the elements of their menus. Make sure it starts with
     # "RabbitVCS::"
     identifier = None
-    
+
     # The label that appears on the menu item. It is up to the subclass to
     # designate it as translatable.
     label = None
-    
+
     # The tooltip for the menu item. It is up to the subclass to designate it as
     # translatable.
     tooltip = ""
-    
+
     # The icon that will appear on the menu item. This can be, say,
-    # "rabbitvcs-something" or gtk.STOCK_SOMETHING
+    # "rabbitvcs-something"
     icon = None
 
     # This is a string that holds the name of the function that is called when
@@ -118,22 +118,22 @@ class MenuItem(object):
     # it cannot find anything, it will set up a function that returns False.
     condition_name = None
     condition_args = ()
-    
+
     def __init__(self, conditions, callbacks):
         """
         Creates a new menu item for constructing the GTK context menu.
         """
-        
+
         self.signals = {}
-        
+
         default_name = MenuItem.make_default_name(self.identifier)
-        
+
         # These flags are used for sanity checks that developers can run to
         # ensure completeness of conditions and callbacks.
-        # See contextmenu.TestMenuItemFunctions() 
+        # See contextmenu.TestMenuItemFunctions()
         self.found_callback = False
         self.found_condition = False
-        
+
         # If no callback name is set, assign the default
         if self.callback_name is None:
             # log.debug("Using default callback name: %s" % default_name)
@@ -148,51 +148,51 @@ class MenuItem(object):
         self.condition = {
             "callback": MenuItem.default_condition,
             "args": self.condition_args
-            }            
+            }
 
         if self.condition_name is None:
             self.condition_name = default_name
-            
+
         condition = self._get_function(conditions, self.condition_name)
-        
+
         if condition:
             self.condition["callback"] = condition
             self.found_condition = True
 #        else:
 #            log.debug("Could not find condition for %s" % self.identifier)
-    
+
     def show(self):
         return self.condition["callback"](*self.condition["args"])
-    
+
     def _get_function(self, object, name):
-        
+
         function = None
-        
+
         if hasattr(object, name):
-            
+
             attr = getattr(object, name)
             if callable(attr):
                 function = attr
-        
+
         return function
-      
+
     def make_magic_id(self, id_magic = None):
         identifier = self.identifier
-        
+
         if id_magic:
             identifier = identifier + "-" + str(id_magic)
-            
+
         return identifier
-      
+
     def make_action(self, id_magic = None):
         """
-        Creates the GTK Action for the menu item. To avoid GTK "helpfully"
+        Creates the Action for the menu item. To avoid GTK "helpfully"
         preventing us from adding duplicates (eg. separators), you can pass in
         a string that will be appended and separated from the actual identifier.
         """
         identifier = self.make_magic_id(id_magic)
-        
-        return gtk.Action(identifier, self.make_label(), None, None)
+
+        return Action(identifier, self.make_label(), self.tooltip, self.icon)
 
     def make_thunar_action(self, id_magic = None):
         identifier = self.make_magic_id(id_magic)
@@ -201,36 +201,50 @@ class MenuItem(object):
             identifier,
             self.make_label(),
             self.tooltip,
-            self.icon 
+            self.icon
         )
-            
+
         return action
 
     def make_gtk_menu_item(self, id_magic = None):
         action = self.make_action(id_magic)
-            
+
         if self.icon:
-            # We use this instead of gtk.Action.set_icon_name because
-            # that method is not available until pygtk 2.16
-            action.set_menu_item_type(gtk.ImageMenuItem)
+            # We use this instead of Gtk.Action.set_icon_name because
+            # that method is not available until pyGtk 2.16
+            action.set_menu_item_type(Gtk.ImageMenuItem)
             menuitem = action.create_menu_item()
-            menuitem.set_image(gtk.image_new_from_icon_name(self.icon, gtk.ICON_SIZE_MENU))
+            menuitem.set_image(Gtk.image_new_from_icon_name(self.icon, Gtk.IconSize.MENU))
         else:
             menuitem = action.create_menu_item()
-            
+
         return menuitem
-        
+
     def make_gtk3_menu_item(self, id_magic = None):
         action = self.make_action(id_magic)
-        
+        menuitem = action.create_menu_item()
+
         if self.icon:
-            menuitem = action.create_menu_item()
-            menuitem.set_image(gtk.Image.new_from_icon_name(self.icon, gtk.IconSize.MENU))
-        else:
-            menuitem = action.create_menu_item()
-            
+            menuitem.set_image(Gtk.Image.new_from_icon_name(self.icon, Gtk.IconSize.MENU))
+
         return menuitem
-    
+
+    def make_thunarx_menu_item(self, id_magic = None):
+        # WARNING: this import is here because it will fail if it is not done
+        # inside a thunar process and therefore can't be in the module proper.
+        # This should only be used for Thunarx-3
+        identifier = self.make_magic_id(id_magic)
+
+        from gi.repository import Thunarx
+        menuitem = Thunarx.MenuItem(
+            name=identifier,
+            label=self.make_label(),
+            tooltip=self.tooltip,
+            icon=self.icon
+        )
+
+        return menuitem
+
     def make_nautilus_menu_item(self, id_magic = None):
         # WARNING: this import is here because it will fail if it is not done
         # inside a nautilus process and therefore can't be in the module proper.
@@ -239,14 +253,6 @@ class MenuItem(object):
         identifier = self.make_magic_id(id_magic)
 
         try:
-            import nautilus
-            menuitem = nautilus.MenuItem(
-                identifier,
-                self.make_label(),
-                self.tooltip,
-                self.icon
-            )
-        except ImportError:
             from gi.repository import Nautilus
             menuitem = Nautilus.MenuItem(
                 name=identifier,
@@ -254,32 +260,38 @@ class MenuItem(object):
                 tip=self.tooltip,
                 icon=self.icon
             )
-            
-                    
+        except ImportError:
+            import nautilus
+            menuitem = nautilus.MenuItem(
+                identifier,
+                self.make_label(),
+                self.tooltip,
+                self.icon
+            )
+
         return menuitem
 
     def make_label(self):
-        label = self.label.replace('_', '__')
-        
+        label = S(self.label).display().replace('_', '__')
         return label
 
 class MenuSeparator(MenuItem):
     identifier = "RabbitVCS::Separator"
     label = SEPARATOR
-       
+
     def make_insensitive(self, menuitem):
         menuitem.set_property("sensitive", False)
-               
+
     def make_thunar_action(self, id_magic = None):
         menuitem = super(MenuSeparator, self).make_thunar_action(id_magic)
         self.make_insensitive(menuitem)
         return menuitem
         # FIXME: I thought that this would work to create separators,
-        # but all I get are black "-"s...		
+        # but all I get are black "-"s...
         # I thought
         #~ identifier = self.make_magic_id(id_magic)
         #~ # This information is not actually used, but is necessary for
-        #~ # the required subclassing of GtkAction.
+        #~ # the required subclassing of Action.
         #~ action = ThunarSeparator(
             #~ identifier,
             #~ self.label,
@@ -287,18 +299,23 @@ class MenuSeparator(MenuItem):
             #~ self.icon,
         #~ )
         #~ return action
-               
+
     # Make separators insensitive
     def make_gtk_menu_item(self, id_magic = None):
-        menuitem = gtk.SeparatorMenuItem()
+        menuitem = Gtk.SeparatorMenuItem()
         menuitem.show()
         return menuitem
-    
+
+    def make_gtk3_menu_item(self, id_magic = None):
+        menuitem = Gtk.SeparatorMenuItem()
+        menuitem.show()
+        return menuitem
+
     def make_nautilus_menu_item(self, id_magic = None):
         menuitem = super(MenuSeparator, self).make_nautilus_menu_item(id_magic)
         self.make_insensitive(menuitem)
         return menuitem
-    
+
 class MenuDebug(MenuItem):
     identifier = "RabbitVCS::Debug"
     label = _("Debug")
@@ -309,11 +326,11 @@ class MenuBugs(MenuItem):
     label = _("Bugs")
     icon = "rabbitvcs-bug"
 
-class MenuDebugShell(MenuItem):
-    identifier = "RabbitVCS::Debug_Shell"
-    label = _("Open Shell")
+class MenuPythonConsole(MenuItem):
+    identifier = "RabbitVCS::Python_Console"
+    label = _("Open Python Console")
     icon = "gnome-terminal"
-    condition_name = "debug" 
+    condition_name = "debug"
 
 class MenuRefreshStatus(MenuItem):
     identifier = "RabbitVCS::Refresh_Status"
@@ -346,7 +363,7 @@ class MenuCheckout(MenuItem):
     label = _("Checkout...")
     tooltip = _("Check out a working copy")
     icon = "rabbitvcs-checkout"
-    
+
 class MenuUpdate(MenuItem):
     identifier = "RabbitVCS::Update"
     label = _("Update")
@@ -383,7 +400,7 @@ class MenuRepoBrowser(MenuItem):
     identifier = "RabbitVCS::Repo_Browser"
     label = _("Repository Browser")
     tooltip = _("Browse a repository tree")
-    icon = gtk.STOCK_FIND
+    icon = "edit-find"
 
 class MenuCheckForModifications(MenuItem):
     identifier = "RabbitVCS::Check_For_Modifications"
@@ -438,7 +455,7 @@ class MenuShowChanges(MenuItem):
     label = _("Show Changes...")
     tooltip = _("Show changes between paths and revisions")
     icon = "rabbitvcs-changes"
-    
+
 class MenuShowLog(MenuItem):
     identifier = "RabbitVCS::Show_Log"
     label = _("Show Log")
@@ -609,7 +626,7 @@ class MenuOpen(MenuItem):
     identifier = "RabbitVCS::Open"
     label = _("Open")
     tooltip = _("Open a file")
-    icon = gtk.STOCK_OPEN
+    icon = "document-open"
     # Not sure why, but it was like this before...
     condition_name = "_open"
     callback_name = "_open"
@@ -618,38 +635,38 @@ class MenuBrowseTo(MenuItem):
     identifier = "RabbitVCS::Browse_To"
     label = _("Browse to")
     tooltip = _("Browse to a file or folder")
-    icon = gtk.STOCK_HARDDISK
+    icon = "drive-harddisk"
 
 class PropMenuRevert(MenuItem):
     identifier = "RabbitVCS::Property_Revert"
     label = _("Revert property")
     icon =  "rabbitvcs-revert"
     tooltip = _("Revert this property to its original state")
-    
+
 class PropMenuRevertRecursive(MenuItem):
     identifier = "RabbitVCS::Property_Revert_Recursive"
     label = _("Revert property (recursive)")
     icon =  "rabbitvcs-revert"
     tooltip = _("Revert this property to its original state (recursive)")
     condition_name = "property_revert"
-    
+
 class PropMenuDelete(MenuItem):
     identifier = "RabbitVCS::Property_Delete"
     label = _("Delete property")
     icon =  "rabbitvcs-delete"
     tooltip = _("Delete this property")
-    
+
 class PropMenuDeleteRecursive(MenuItem):
     identifier = "RabbitVCS::Property_Delete_Recursive"
     label = _("Delete property (recursive)")
     icon =  "rabbitvcs-delete"
     tooltip = _("Delete this property (recursive)")
     condition_name = "property_delete"
-    
+
 class PropMenuEdit(MenuItem):
     identifier = "RabbitVCS::Property_Edit"
     label = _("Edit details")
-    icon = gtk.STOCK_EDIT
+    icon = "rabbitvcs-editprops"
     tooltip = _("Show and edit property details")
 
 class MenuInitializeRepository(MenuItem):
@@ -700,7 +717,7 @@ class MenuStage(MenuItem):
     identifier = "RabbitVCS::Stage"
     label = _("Stage")
     icon = "rabbitvcs-add"
-    
+
 class MenuUnstage(MenuItem):
     identifier = "RabbitVCS::Unstage"
     label = _("Unstage")
@@ -720,17 +737,17 @@ def get_ignore_list_items(paths):
 
     """
     ignore_items = []
-    
+
     # Used to weed out duplicate menu items
     added_ignore_labels = []
-    
+
     # These are ignore-by-filename items
     ignorebyfilename_index = 0
     for path in paths:
         basename = os.path.basename(path)
         if basename not in added_ignore_labels:
             key = "IgnoreByFileName%s" % str(ignorebyfilename_index)
-            
+
             class MenuIgnoreFilenameClass(MenuItem):
                 identifier = "RabbitVCS::%s" % key
                 label = basename
@@ -739,17 +756,17 @@ def get_ignore_list_items(paths):
                 callback_args = (path)
                 condition_name = "ignore_by_filename"
                 condition_args = (path)
-            
+
             ignore_items.append((MenuIgnoreFilenameClass, None))
 
     # These are ignore-by-extension items
     ignorebyfileext_index = 0
     for path in paths:
-        extension = rabbitvcs.util.helper.get_file_extension(path)
-        
+        extension = helper.get_file_extension(path)
+
         ext_str = "*%s"%extension
         if ext_str not in added_ignore_labels:
-            
+
             class MenuIgnoreFileExtClass(MenuItem):
                 identifier = "RabbitVCS::%s" % key
                 label = ext_str
@@ -758,39 +775,50 @@ def get_ignore_list_items(paths):
                 callback_args = (path, extension)
                 condition_name = "ignore_by_file_extension"
                 condition_args = (path, extension)
-            
+
             ignore_items.append((MenuIgnoreFileExtClass, None))
 
     return ignore_items
 
-class RabbitVCSAction(gtk.Action):
-    """
-    Sub-classes gtk.Action so that we can have submenus
-    """
-
-    __gtype_name__ = "RabbitVCSAction"
-
-    def __init__(self, name, label, tooltip, stock_id):
-        gtk.Action.__init__(self, name, label, tooltip, stock_id)
-        self.sub_actions = None
-        self.stock_id = stock_id
+class Action(object):
+    def __init__(self, name, label, tooltip, icon_name):
+        self.name = name
+        self.label = label
+        self.tooltip = tooltip
+        self.icon_name = icon_name
 
     def __repr__(self):
         return self.get_name()
 
+    def create_menu_item(self):
+        item = Gtk.ImageMenuItem()
+        if self.label:
+            item.set_label(self.label)
+        if self.tooltip:
+            item.set_tooltip_text(self.tooltip)
+        if self.icon_name:
+            item.set_image(Gtk.Image.new_from_icon_name(self.icon_name, Gtk.IconSize.MENU))
+        return item
+
+class RabbitVCSAction(Action):
+    """
+    Sub-classes Action so that we can have submenus.
+    This is needed for context menus that use Gtk actions
+    """
+
+    __gtype_name__ = "RabbitVCSAction"
+
+    def __init__(self, name, label, tooltip, icon_name):
+        Action.__init__(self, name, label, tooltip, icon_name)
+        self.sub_actions = None
+
     def set_sub_actions(self, sub_actions):
         self.sub_actions = sub_actions
 
-    def do_create_menu_item(self):
-        menu_item = gtk.ImageMenuItem()
-        if self.stock_id:
-            try:
-                self.set_icon_name(self.stock_id)
-            except AttributeError as e:
-                menu_item.set_image(gtk.image_new_from_icon_name(self.stock_id, gtk.ICON_SIZE_MENU))
-
+    def create_menu_item(self):
+        menu_item = super(RabbitVCSAction, self).create_menu_item()
         if self.sub_actions is not None:
-            menu = gtk.Menu()
+            menu = Gtk.Menu()
             menu_item.set_submenu(menu)
 
             for sub_action in self.sub_actions:
@@ -803,6 +831,6 @@ class RabbitVCSAction(gtk.Action):
 # FIXME: apparently it's possible to get real GtkSeparators in a Thunar
 # menu, but this doesn't seem to work.
 class ThunarSeparator(RabbitVCSAction):
-		
-	def do_create_menu_item(self):
-		return gtk.SeparatorMenuItem()
+
+    def create_menu_item(self):
+        return Gtk.SeparatorMenuItem()
