@@ -1,22 +1,22 @@
 from __future__ import absolute_import
 #
-# This is an extension to the Nautilus file manager to allow better 
+# This is an extension to the Nautilus file manager to allow better
 # integration with the Subversion source control system.
-# 
+#
 # Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
 # Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
 # Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
-# 
+#
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # RabbitVCS is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
 #
@@ -24,13 +24,14 @@ from __future__ import absolute_import
 import rabbitvcs.util.helper
 import rabbitvcs.vcs
 
-class BranchEntry:
+
+class BranchEntry(object):
     def __init__(self, name, tracking=False, revision="", message=""):
         self.name = name
         self.tracking = tracking
         self.revision = revision
         self.message = message
-    
+
     def __str__(self):
         tracking = self.tracking and " (tracking)" or ""
         return "<BranchEntry %s:%s%s>" % (self.name, self.revision, tracking)
