@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+from __future__ import print_function
 #
 # test/pull.py
 #
@@ -19,7 +21,7 @@ DIR = "pull"
 if options.cleanup:
     rmtree(DIR, ignore_errors=True)
 
-    print "pull.py clean"
+    print("pull.py clean")
 else:
     if os.path.isdir(DIR):
         raise SystemExit("This test script has already been run.  Please call this script with --cleanup to start again")
@@ -28,4 +30,4 @@ else:
     g.remote_add("origin", "git://github.com/adamplumb/gittyup.git")
     g.pull("origin", "master")
 
-    print "pull.py pass"
+    print("pull.py pass")
