@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 #
 # util.py
 #
@@ -5,7 +6,7 @@
 import os
 
 def touch(fname, times = None):
-    with file(fname, 'a'):
+    with open(fname, 'a'):
         os.utime(fname, times)
 
 def change(path):
