@@ -1,21 +1,21 @@
 #
-# This is an extension to the Nautilus file manager to allow better 
+# This is an extension to the Nautilus file manager to allow better
 # integration with the Subversion source control system.
-# 
+#
 # Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
 # Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
 # Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
-# 
+#
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # RabbitVCS is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
 #
@@ -30,7 +30,7 @@ from datetime import datetime
 
 from mercurial import commands, ui, hg
 
-import rabbitvcs.util.helper
+from rabbitvcs.util.strings import S
 
 import rabbitvcs.vcs
 import rabbitvcs.vcs.status
@@ -38,14 +38,14 @@ import rabbitvcs.vcs.log
 import rabbitvcs.vcs.mercurial.util
 from rabbitvcs.vcs.branch import BranchEntry
 from rabbitvcs.util.log import Log
-import six
 
 log = Log("rabbitvcs.vcs.mercurial")
 
 from rabbitvcs import gettext
 _ = gettext.gettext
 
-class Revision:
+
+class Revision(object):
     """
     Implements a simple revision object as a wrapper around the gittyup revision
     object.  This allows us to provide a standard interface to the object data.
@@ -54,34 +54,34 @@ class Revision:
     def __init__(self, kind, value=None):
         self.kind = kind.upper()
         self.value = value
-        
+
         if self.kind == "HEAD":
             self.value = "HEAD"
-        
+
         self.is_revision_object = True
 
-    def __unicode__(self):
+    def __str__(self):
         if self.value:
-            return six.text_type(self.value)
-        else:
-            return self.kind
-            
+            return S(self.value)
+        return S(self.kind)
+
+    def __unicode__(self):
+        return self.__str__().unicode()
+
     def short(self):
         if self.value:
-            return six.text_type(self.value)[0:7]
+            return S(self.value)[0:7]
         else:
             return self.kind
 
-    def __str__(self):
-        return self.__unicode__()
-
     def __repr__(self):
-        return self.__unicode__()
+        return self.__str__()
 
     def primitive(self):
         return self.value
 
-class Mercurial:
+
+class Mercurial(object):
     STATUS = {
         "normal":       "C",
         "added":        "A",
@@ -90,7 +90,7 @@ class Mercurial:
         "untracked":    "?",
         "missing":      "!"
     }
-    
+
     STATUS_REVERSE = {
         "C":       "normal",
         "A":        "added",
@@ -113,7 +113,7 @@ class Mercurial:
         "added",
         "removed"
     ]
-    
+
     STATUSES_FOR_STAGE = [
         "untracked"
     ]
@@ -146,11 +146,11 @@ class Mercurial:
         while path_to_check != "/" and path_to_check != "":
             if os.path.isdir(os.path.join(path_to_check, ".hg")):
                 return path_to_check
-            
+
             path_to_check = os.path.split(path_to_check)[0]
-        
+
         return None
-    
+
     def get_relative_path(self, path):
         if path == self.repository_path:
             return ""
@@ -159,14 +159,14 @@ class Mercurial:
 
     def get_absolute_path(self, path):
         return os.path.join(self.repository_path, path).rstrip("/")
-    
+
     def statuses(self, path, recurse=True, invalidate=False):
         mercurial_statuses = self.repository.status(clean=True, unknown=True)
 
         # the status method returns a series of tuples filled with files matching
         # the statuses below
         tuple_order = ["modified", "added", "removed", "missing", "unknown", "ignored", "clean"]
-        
+
         # go through each tuple (each of which has a defined status), and
         # generate a flat list of rabbitvcs statuses
         statuses = []
@@ -176,18 +176,18 @@ class Mercurial:
             content = tuple_order[index]
             for item in status_tuple:
                 st_path = self.get_absolute_path(item)
-                
+
                 rabbitvcs_status = rabbitvcs.vcs.status.MercurialStatus({
                     "path": st_path,
                     "content": content
                 })
                 statuses.append(rabbitvcs_status)
-                
+
                 # determine the statuses of the parent folders
                 dir_content = content
                 if content in self.STATUSES_FOR_REVERT:
                     dir_content = "modified"
-                
+
                 path_to_check = os.path.dirname(st_path)
                 while True:
                     if path_to_check not in directories or directories[path_to_check] not in self.STATUSES_FOR_COMMIT:
@@ -197,7 +197,7 @@ class Mercurial:
                         })
                         statuses.append(rabbitvcs_status)
                         directories[path_to_check] = dir_content
-                    
+
                     if path_to_check == "" or path_to_check == self.repository_path:
                         break
 
@@ -206,7 +206,7 @@ class Mercurial:
             index += 1
 
         return statuses
-    
+
     def status(self, path, summarize=True, invalidate=False):
         all_statuses = self.statuses(path, invalidate=invalidate)
 
@@ -241,7 +241,7 @@ class Mercurial:
     def is_versioned(self, path):
         if self.is_working_copy(path):
             return True
-       
+
         st = self.status(path)
         try:
             return st.is_versioned()
@@ -250,14 +250,14 @@ class Mercurial:
             return False
 
         return False
-    
+
     def is_locked(self, path):
         return False
 
     def get_items(self, paths, statuses=[]):
         if paths is None:
             return []
-        
+
         items = []
         for path in paths:
             st = self.statuses(path, invalidate=True)
@@ -269,7 +269,7 @@ class Mercurial:
                     items.append(st_item)
 
         return items
-    
+
     #
     # Actions
     #
