@@ -36,10 +36,12 @@ import pysvn
 import rabbitvcs.vcs
 import rabbitvcs.vcs.status
 import rabbitvcs.vcs.log
-import rabbitvcs.util.helper
+from rabbitvcs.util import helper
 from rabbitvcs.util.log import Log
-from six.moves import map
+from rabbitvcs.util.decorators import structure_map
+from rabbitvcs.util.strings import *
 import six
+from six.moves import map
 from six.moves import range
 
 log = Log("rabbitvcs.vcs.svn")
@@ -50,7 +52,23 @@ _ = gettext.gettext
 # Extra "action" for "commit completed"
 commit_completed = "commit_completed"
 
-class Revision:
+@structure_map
+def pure_unicode(obj):
+    """
+    Map object string subclass components to real unicode type.
+
+    Pysvn (using PyCXX) does not like string subclasses as arguments and
+    requires unsubclassed string/unicode objects. In addition, it fails on
+    Python2 non-ascii byte strings when converting them to utf-8.
+    This function is called each time there is a risk of passing string or
+    unicode objects to the pysvn API.
+    """
+    if isinstance(obj, (six.string_types, six.text_type)):
+        obj = S(obj).unicode()
+    return obj
+
+
+class Revision(object):
     """
     Implements a simple revision object as a wrapper around the pysvn revision
     object.  This allows us to provide a standard interface to the object data.
@@ -68,7 +86,7 @@ class Revision:
     }
 
     def __init__(self, kind, value=None):
-        self.kind = six.text_type(kind).lower()
+        self.kind = S(kind).lower()
         self.value = value
         self.is_revision_object = True
 
@@ -93,25 +111,25 @@ class Revision:
         except Exception as e:
             log.exception(e)
 
-    def __unicode__(self):
+    def __str__(self):
         if self.value:
-            return six.text_type(self.value)
-        else:
-            return self.kind
+            return S(self.value)
+        return S(self.kind)
+
+    def __unicode__(self):
+        return self.__str__().unicode()
 
     def short(self):
-        return self.__unicode__()
-
-    def __str__(self):
-        return self.__unicode__()
+        return self.__str__()
 
     def __repr__(self):
-        return self.__unicode__()
+        return self.__str__()
 
     def primitive(self):
         return self.__revision
 
-class SVN:
+
+class SVN(object):
     """
 
     """
@@ -194,6 +212,7 @@ class SVN:
         pysvn.wc_notify_action.skip:                    _("Skipped"),
         pysvn.wc_notify_action.update_delete:           _("Deleted"),
         pysvn.wc_notify_action.update_add:              _("Added"),
+        pysvn.wc_notify_action.update_started:          _("Updating"),
         pysvn.wc_notify_action.update_update:           _("Updated"),
         pysvn.wc_notify_action.update_completed:        _("Completed"),
         pysvn.wc_notify_action.update_external:         _("External"),
@@ -247,12 +266,13 @@ class SVN:
         Look up the status for path.
 
         """
-        
-        if path in self.cache:
+
+        spath = S(path)
+        if spath in self.cache:
             if invalidate:
-                del self.cache[path]
+                del self.cache[spath]
             else:
-                return self.cache.find_path_statuses(path)
+                return self.cache.find_path_statuses(spath)
 
         on_error = rabbitvcs.vcs.status.Status.status_unknown(path)
 
@@ -260,7 +280,7 @@ class SVN:
             return [on_error]
 
         try:
-            pysvn_statuses = self.client.status(path,
+            pysvn_statuses = self.client_status(path,
                                                 depth=pysvn.depth.infinity,
                                                 update=update)
             if not len(pysvn_statuses):
@@ -272,43 +292,48 @@ class SVN:
                 for st in pysvn_statuses:
                     # If not recursing, only return the item in question (if a file)
                     # or items directly under the path (if a directory)
-                    cmp_path = os.path.join(path, os.path.basename(st.path))
-                    if not recurse and cmp_path != st.path and st.path != path:
+                    st_path = S(st.path)
+                    cmp_path = os.path.join(path, os.path.basename(st_path))
+                    if not recurse and cmp_path != st_path and st_path != path:
                         continue
 
                     rabbitvcs_status = rabbitvcs.vcs.status.SVNStatus(st)
-                    self.cache[st.path] = rabbitvcs_status
+                    self.cache[st_path] = rabbitvcs_status
                     statuslist.append(rabbitvcs_status)
-                    
+
                 return statuslist
         except pysvn.ClientError as ex:
             # TODO: uncommenting these might not be a good idea
             #~ traceback.print_exc()
-            log.debug("Exception occured in SVN.status() for %s" % path)
+            log.debug("Exception occurred in SVN.status() for %s" % path)
             log.exception(ex)
             return [on_error]
 
     def client_info(self, path):
         if islink(path):
             path = realpath(path)
-        return self.client.info(path)
+        return self.client.info(pure_unicode(path))
+
+    def client_status(self, *args, **kwargs):
+        return self.client.status(*pure_unicode(args), **pure_unicode(kwargs))
 
     def find_repository_path(self, path):
         path_to_check = path
         while path_to_check != "/" and path_to_check != "":
             if os.path.isdir(os.path.join(path_to_check, ".svn")):
                 return path_to_check
-            
+
             path_to_check = os.path.split(path_to_check)[0]
-        
+
         return None
 
     def status(self, path, summarize=True, invalidate=False):
-        if path in self.cache:
+        spath = S(path)
+        if spath in self.cache:
             if invalidate:
-                del self.cache[path]
+                del self.cache[spath]
             else:
-                st = self.cache[path]
+                st = self.cache[spath]
                 if summarize:
                     st.summary = st.single
                 return st
@@ -318,10 +343,10 @@ class SVN:
         if summarize:
             path_status = None
             for st in all_statuses:
-                if st.path == path:
+                if S(st.path) == spath:
                     path_status = st
                     break
-            
+
             path_status.make_summary(all_statuses)
         else:
             path_status = all_statuses[0]
@@ -352,8 +377,9 @@ class SVN:
     def is_in_a_or_a_working_copy(self, path):
         if self.is_working_copy(path):
             return True
-
-        return (self.find_repository_path(os.path.split(path)[0]) != "")
+        if self.find_repository_path(os.path.split(path)[0]):
+            return True
+        return False
 
     def is_versioned(self, path):
         if self.is_working_copy(path):
@@ -366,7 +392,7 @@ class SVN:
                     return True
             except Exception as e:
                 log.exception("is_versioned exception for %s" % path)
-                
+
             return False
 
     def is_status(self, path, status_kind):
@@ -390,6 +416,7 @@ class SVN:
 
     def is_locked(self, path):
         is_locked = False
+        path = pure_unicode(path)
         try:
             is_locked = self.client.info2(path, recurse=False)[0][1].lock is not None
         except pysvn.ClientError as e:
@@ -412,7 +439,7 @@ class SVN:
         @return:            A list of statuses
 
         """
-        
+
         items = []
 
         for path in paths:
@@ -427,7 +454,7 @@ class SVN:
     def get_remote_updates(self, paths):
         if paths is None:
             return []
-        
+
         items = []
         for path in paths:
             try:
@@ -437,7 +464,7 @@ class SVN:
                 continue
 
             for st in sts:
-                        
+
                 if st.remote_content is None and st.remote_metadata is None:
                     continue
 
@@ -471,10 +498,11 @@ class SVN:
         if not path:
             return ""
 
-        info = self.client_info(path)
         returner = ""
+
         try:
-            returner = info["url"].encode('latin1')
+            info = self.client_info(path)
+            returner = S(info["url"], IDENTITY_ENCODING).bytes()
         except Exception as e:
             log.exception(e)
 
@@ -493,9 +521,10 @@ class SVN:
 
         """
 
-        info = self.client.info2(path, recurse=False)
         returner = ""
+
         try:
+            info = self.client.info2(pure_unicode(path), recurse=False)
             returner = info[0][1]["repos_root_URL"]
         except Exception as e:
             log.exception(e)
@@ -657,16 +686,16 @@ class SVN:
 
         try:
             self.client.propset(
-                prop_name,
-                props,
-                path,
+                pure_unicode(prop_name),
+                pure_unicode(props),
+                pure_unicode(path),
                 recurse=recurse
             )
             return True
         except pysvn.ClientError as e:
-            log.exception("pysvn.ClientError exception in svn.py propset() for %s" % path)
+            log.exception("pysvn.ClientError exception in svn.py propset() for %s" % S(path).display())
         except TypeError as e:
-            log.exception("TypeError exception in svn.py propset() %s" % path)
+            log.exception("TypeError exception in svn.py propset() %s" % S(path).display())
 
         return False
 
@@ -681,6 +710,7 @@ class SVN:
         @return:        A dictionary of properties.
 
         """
+        path = pure_unicode(path)
         if rev:
             returner = self.client.proplist(path, revision=rev)
         else:
@@ -709,7 +739,8 @@ class SVN:
 
         """
 
-        path = self.get_versioned_path(path)
+        path = self.get_versioned_path(pure_unicode(path))
+        prop_name = pure_unicode(prop_name)
         try:
             if rev:
                 returner = self.client.propget(
@@ -756,15 +787,15 @@ class SVN:
         returner = False
         try:
             self.client.propdel(
-                prop_name,
-                path,
+                pure_unicode(prop_name),
+                pure_unicode(path),
                 recurse=recurse
             )
             returner = True
         except pysvn.ClientError as e:
-            log.exception("pysvn.ClientError exception in svn.py propdel() for %s" % path)
+            log.exception("pysvn.ClientError exception in svn.py propdel() for %s" % S(path).display())
         except TypeError as e:
-            log.exception("TypeError exception in svn.py propdel() %s" % path)
+            log.exception("TypeError exception in svn.py propdel() %s" % S(path).display())
 
         return returner
 
@@ -841,8 +872,10 @@ class SVN:
         if rev is None:
             rev = self.revision("head")
 
-        self.client.revpropset(prop_name, prop_value, url,
-            revision=rev.primitive())
+        self.client.revpropset(pure_unicode(prop_name),
+                               pure_unicode(prop_value),
+                               pure_unicode(url),
+                               revision=rev.primitive())
 
     def revproplist(self, url, rev=None):
         """
@@ -862,7 +895,7 @@ class SVN:
         if rev is None:
             rev = self.revision("head")
 
-        return self.client.revproplist(url, rev.primitive())[1]
+        return self.client.revproplist(pure_unicode(url), rev.primitive())[1]
 
     def revpropget(self, url, prop_name, rev=None):
         """
@@ -886,8 +919,8 @@ class SVN:
             rev = self.revision("head")
 
         return self.client.revpropget(
-            prop_name,
-            url,
+            pure_unicode(prop_name),
+            pure_unicode(url),
             revision=rev.primitive()
         )
 
@@ -913,8 +946,8 @@ class SVN:
             rev = self.revision("head")
 
         return self.client.revpropdel(
-            prop_name,
-            url,
+            pure_unicode(prop_name),
+            pure_unicode(url),
             revision=rev.primitive(),
             force=force
         )
@@ -997,7 +1030,7 @@ class SVN:
 
         """
 
-        return self.client.add(paths, recurse)
+        return self.client.add(pure_unicode(paths), recurse)
 
     def add_backwards(self, path):
         """
@@ -1008,7 +1041,7 @@ class SVN:
         @param path: the path to add to version control
         @type path: string
         """
-        head, tail = path,""
+        head, tail = pure_unicode(path),""
         tails = list()
 
         # We need to add backwards-recursively, since patch could create
@@ -1040,8 +1073,8 @@ class SVN:
         @param  revision: A pysvn.Revision object.
 
         """
-        src = rabbitvcs.util.helper.urlize(src)
-        dest = rabbitvcs.util.helper.urlize(dest)
+        src = helper.urlize(pure_unicode(src))
+        dest = helper.urlize(pure_unicode(dest))
         return self.client.copy(src, dest, revision.primitive())
 
     def copy_all(self, sources, dest_url_or_path, copy_as_child=False,
@@ -1066,8 +1099,9 @@ class SVN:
         @param  ignore_externals: Omit externals
 
         """
-        
-        return self.client.copy2(sources, dest_url_or_path, copy_as_child,
+
+        return self.client.copy2(pure_unicode(sources),
+            pure_unicode(dest_url_or_path), copy_as_child,
             make_parents, None, ignore_externals)
 
     def checkout(self, url, path, recurse=True, revision=Revision("head"),
@@ -1093,10 +1127,11 @@ class SVN:
 
         """
 
-        url = rabbitvcs.util.helper.urlize(url)
+        url = helper.urlize(pure_unicode(url))
 
-        return self.client.checkout(url, path, recurse=recurse,
-            revision=revision.primitive(), ignore_externals=ignore_externals)
+        return self.client.checkout(url, pure_unicode(path),
+            recurse=recurse, revision=revision.primitive(),
+            ignore_externals=ignore_externals)
 
     def cleanup(self, path):
         """
@@ -1107,7 +1142,7 @@ class SVN:
 
         """
 
-        return self.client.cleanup(path)
+        return self.client.cleanup(pure_unicode(path))
 
     def revert(self, paths):
         """
@@ -1118,7 +1153,7 @@ class SVN:
 
         """
 
-        return self.client.revert(paths)
+        return self.client.revert(pure_unicode(paths))
 
     def commit(self, paths, log_message="", recurse=False, keep_locks=False):
         """
@@ -1137,7 +1172,7 @@ class SVN:
         @param  keep_locks: Whether or not to keep locks on commit.
 
         """
-        
+
         kwargs = {"keep_locks": keep_locks}
         try:
             # Simply setting recurse=False will not stop child files from getting
@@ -1147,8 +1182,9 @@ class SVN:
             kwargs["depth"] = (recurse and pysvn.depth.infinity or pysvn.depth.empty)
         except AttributeError:
             kwargs["recurse"] = recurse
-        
-        retval = self.client.checkin(paths, log_message, **kwargs)
+
+        retval = self.client.checkin(pure_unicode(paths),
+                                     pure_unicode(log_message), **kwargs)
         dummy_commit_dict = {
             "revision": retval,
             "action": rabbitvcs.vcs.svn.commit_completed
@@ -1177,40 +1213,40 @@ class SVN:
 
         """
 
-        log = self.client.log(url_or_path, revision_start.primitive(),
-            revision_end.primitive(), discover_changed_paths,
-            strict_node_history, limit)
+        log = self.client.log(pure_unicode(url_or_path),
+            revision_start.primitive(), revision_end.primitive(),
+            discover_changed_paths, strict_node_history, limit)
 
         returner = []
         for item in log:
             revision = Revision(pysvn.opt_revision_kind.number, item.revision.number)
             date = datetime.fromtimestamp(item.date) if hasattr(item, "date") else datetime(1900, 1, 1)
-            
+
             author = _("(no author)")
             if hasattr(item, "author"):
-                author = item["author"]        
+                author = item["author"]
 
             message = ""
             if hasattr(item, "message"):
                 message = item["message"]
-        
+
             changed_paths = []
             for changed_path in item.changed_paths:
                 copy_from_rev = ""
                 if hasattr(changed_path.copyfrom_revision, "number"):
                     copy_from_rev = self.revision("number", changed_path.copyfrom_revision.number)
-            
+
                 copy_from_path = ""
                 if hasattr(changed_path, "copy_from_path"):
                     copy_from_path = changed_path.copy_from_path
-                    
+
                 changed_paths.append(rabbitvcs.vcs.log.LogChangedPath(
                     changed_path.path,
                     changed_path.action,
                     copy_from_path,
-                    copy_from_rev                    
+                    copy_from_rev
                 ))
-        
+
             returner.append(rabbitvcs.vcs.log.Log(
                 date,
                 revision,
@@ -1219,7 +1255,7 @@ class SVN:
                 changed_paths,
                 None
             ))
-        
+
         return returner
 
     def export(self, src_url_or_path, dest_path, revision=Revision("head"),
@@ -1246,7 +1282,8 @@ class SVN:
 
         """
 
-        self.client.export(src_url_or_path, dest_path, force,
+        self.client.export(pure_unicode(src_url_or_path),
+            pure_unicode(dest_path), force,
             revision.primitive(), native_eol, ignore_externals, recurse)
 
     def import_(self, path, url, log_message, ignore=False):
@@ -1268,8 +1305,10 @@ class SVN:
 
         """
 
-        url = rabbitvcs.util.helper.urlize(url)
-        return self.client.import_(path, url, log_message, ignore)
+        url = helper.urlize(url)
+        return self.client.import_(pure_unicode(path),
+                                   pure_unicode(url),
+                                   pure_unicode(log_message), ignore)
 
     def lock(self, url_or_path, lock_comment, force=False):
 
@@ -1287,7 +1326,8 @@ class SVN:
 
         """
 
-        return self.client.lock(url_or_path, lock_comment, force)
+        return self.client.lock(pure_unicode(url_or_path),
+                                pure_unicode(lock_comment), force)
 
     def relocate(self, from_url, to_url, path, recurse=True):
 
@@ -1304,9 +1344,9 @@ class SVN:
         @param  path: The path of the local working copy
 
         """
-        
-        from_url = rabbitvcs.util.helper.urlize(from_url)
-        to_url = rabbitvcs.util.helper.urlize(to_url)
+
+        from_url = helper.urlize(pure_unicode(from_url))
+        to_url = helper.urlize(pure_unicode(to_url))
         return self.client.relocate(from_url, to_url, path, recurse)
 
     def move(self, src_url_or_path, dest_url_or_path):
@@ -1322,6 +1362,8 @@ class SVN:
 
         """
 
+        src_url_or_path = pure_unicode(src_url_or_path)
+        dest_url_or_path = pure_unicode(dest_url_or_path)
         if hasattr(self.client, "move2"):
             return self.client.move2([src_url_or_path], dest_url_or_path)
         else:
@@ -1348,7 +1390,8 @@ class SVN:
 
         """
 
-        return self.client.move2(sources, dest_url_or_path,
+        return self.client.move2(pure_unicode(sources),
+            pure_unicode(dest_url_or_path),
             move_as_child=move_as_child, make_parents=make_parents)
 
     def remove(self, url_or_path, force=False, keep_local=False):
@@ -1367,7 +1410,8 @@ class SVN:
 
         """
 
-        return self.client.remove(url_or_path, force, keep_local)
+        return self.client.remove(pure_unicode(url_or_path),
+                                  force, keep_local)
 
     def revert(self, paths, recurse=False):
         """
@@ -1381,7 +1425,7 @@ class SVN:
 
         """
 
-        return self.client.revert(paths, recurse)
+        return self.client.revert(pure_unicode(paths), recurse)
 
     def resolve(self, path, recurse=True):
         """
@@ -1395,7 +1439,7 @@ class SVN:
 
         """
 
-        return self.client.resolved(path, recurse)
+        return self.client.resolved(pure_unicode(path), recurse)
 
     def switch(self, path, url, revision=Revision("head")):
         """
@@ -1412,8 +1456,8 @@ class SVN:
 
         """
 
-        url = rabbitvcs.util.helper.urlize(url)
-        return self.client.switch(path, url, revision.primitive())
+        url = helper.urlize(pure_unicode(url))
+        return self.client.switch(pure_unicode(path), url, revision.primitive())
 
     def unlock(self, path, force=False):
         """
@@ -1427,7 +1471,7 @@ class SVN:
 
         """
 
-        return self.client.unlock(path, force)
+        return self.client.unlock(pure_unicode(path), force)
 
     def update(self, path, recurse=True, revision=Revision("head"),
             ignore_externals=False):
@@ -1448,8 +1492,9 @@ class SVN:
 
         """
 
-        return self.client.update(path, recurse, revision.primitive(),
-            ignore_externals)
+        return self.client.update(pure_unicode(path),
+                                  recurse, revision.primitive(),
+                                  ignore_externals)
 
     def annotate(self, url_or_path, from_revision=Revision("number", 1),
             to_revision=Revision("head")):
@@ -1467,8 +1512,9 @@ class SVN:
 
         """
 
-        return self.client.annotate(url_or_path, from_revision.primitive(),
-            to_revision.primitive())
+        return self.client.annotate(pure_unicode(url_or_path),
+                                    from_revision.primitive(),
+                                    to_revision.primitive())
 
     def merge_ranges(self, source, ranges_to_merge, peg_revision,
             target_wcpath, notice_ancestry=False, force=False, dry_run=False,
@@ -1503,10 +1549,10 @@ class SVN:
         TODO: Will firm up the parameter documentation later
 
         """
-        return self.client.merge_peg2(source,
+        return self.client.merge_peg2(pure_unicode(source),
                                       ranges_to_merge,
                                       peg_revision.primitive(),
-                                      target_wcpath,
+                                      pure_unicode(target_wcpath),
                                       notice_ancestry=notice_ancestry,
                                       force=force,
                                       dry_run=dry_run,
@@ -1520,7 +1566,7 @@ class SVN:
         return hasattr(self.client, "merge_peg2")
 
     def merge_trees(self, url_or_path1, revision1, url_or_path2, revision2,
-            local_path, force=False, recurse=True, record_only=False):
+            local_path, force=False, recurse=True, dry_run=False, record_only=False):
         """
         Merge two trees into one.
 
@@ -1545,6 +1591,9 @@ class SVN:
         @type   recurse: boolean
         @param  recurse: Merge children recursively
 
+        @type   dry_run: boolean
+        @param  dry_run: Do a test/dry run or not
+
         @type   record_only: boolean
         @param  record_only: unsure
 
@@ -1552,11 +1601,17 @@ class SVN:
 
         """
 
-        url_or_path1 = rabbitvcs.util.helper.urlize(url_or_path1)
-        url_or_path2 = rabbitvcs.util.helper.urlize(url_or_path2)
-        return self.client.merge(url_or_path1, revision1.primitive(),
-            url_or_path2, revision2.primitive(), local_path, force, recurse,
-            record_only)
+        url_or_path1 = helper.urlize(pure_unicode(url_or_path1))
+        url_or_path2 = helper.urlize(pure_unicode(url_or_path2))
+        return self.client.merge(url_or_path1,
+                                 revision1.primitive(),
+                                 url_or_path2,
+                                 revision2.primitive(),
+                                 pure_unicode(local_path),
+                                 force = force,
+                                 recurse = recurse,
+                                 dry_run = dry_run,
+                                 record_only = record_only)
 
     def has_merge_reintegrate(self):
         """
@@ -1583,8 +1638,10 @@ class SVN:
 
         """
 
-        return self.client.merge_reintegrate(url_or_path, revision.primitive(),
-            local_path, dry_run)
+        return self.client.merge_reintegrate(pure_unicode(url_or_path),
+                                             revision.primitive(),
+                                             pure_unicode(local_path),
+                                             dry_run = dry_run)
 
 
     def diff(self, tmp_path, url_or_path, revision1, url_or_path2, revision2,
@@ -1622,9 +1679,12 @@ class SVN:
 
         """
 
-        return self.client.diff(tmp_path, url_or_path, revision1.primitive(),
-            url_or_path2, revision2.primitive(), recurse, ignore_ancestry,
-            diff_deleted, ignore_content_type)
+        return self.client.diff(pure_unicode(tmp_path),
+                                pure_unicode(url_or_path),
+                                revision1.primitive(),
+                                pure_unicode(url_or_path2),
+                                revision2.primitive(), recurse, ignore_ancestry,
+                                diff_deleted, ignore_content_type)
 
     def diff_summarize(self, url_or_path1, revision1, url_or_path2, revision2,
             recurse=True, ignore_ancestry=False):
@@ -1654,13 +1714,16 @@ class SVN:
 
         """
 
-        return self.client.diff_summarize(url_or_path1, revision1.primitive(),
-            url_or_path2, revision2.primitive(), recurse, ignore_ancestry)
+        return self.client.diff_summarize(pure_unicode(url_or_path1),
+                                          revision1.primitive(),
+                                          pure_unicode(url_or_path2),
+                                          revision2.primitive(),
+                                          recurse, ignore_ancestry)
 
     def list(self, url_or_path, revision=Revision("HEAD"), recurse=True):
-        url_or_path = rabbitvcs.util.helper.urlize(url_or_path)
-        return self.client.list(url_or_path, revision=revision.primitive(),
-            recurse=recurse)
+        url_or_path = helper.urlize(url_or_path)
+        return self.client.list(pure_unicode(url_or_path),
+                                revision=revision.primitive(), recurse=recurse)
 
     def mkdir(self, url_or_path, log_message):
         """
@@ -1674,7 +1737,8 @@ class SVN:
 
         """
 
-        return self.client.mkdir(url_or_path, log_message)
+        return self.client.mkdir(pure_unicode(url_or_path),
+                                 pure_unicode(log_message))
 
     def apply_patch(self, patch_file, base_dir):
         """
@@ -1690,7 +1754,7 @@ class SVN:
 
         any_failures = False
 
-        for file, success, rej_file in rabbitvcs.util.helper.parse_patch_output(patch_file, base_dir):
+        for file, success, rej_file in helper.parse_patch_output(patch_file, base_dir):
 
             fullpath = os.path.join(base_dir, file)
 
@@ -1714,7 +1778,7 @@ class SVN:
                     "path" : rej_file,
                     "action" : _("Rejected Patch"),
                     "mime_type" : None
-                            }
+                }
 
             if self.client.callback_notify:
                 self.client.callback_notify(event_dict)
@@ -1748,29 +1812,3 @@ class SVN:
             return True
 
         return False
-
-    def is_version_greater_than(self, version):
-        """
-        @type   version: tuple
-        @param  version: A version tuple to compare pysvn's version to
-        """
-
-        if version[0] < pysvn.version[0]:
-            return True
-
-        if ((version[0] == pysvn.version[0])
-                and (version[1] < pysvn.version[1])):
-            return True
-
-        if ((version[0] == pysvn.version[0])
-                and (version[1] == pysvn.version[1])
-                and (version[2] < pysvn.version[2])):
-            return True
-
-        if ((version[0] == pysvn.version[0])
-                and (version[1] == pysvn.version[1])
-                and (version[2] == pysvn.version[2])
-                and (version[3] < pysvn.version[3])):
-            return True
-
-        return False
